#!/usr/bin/ruby
# $Id: fixup_datums.rb 8198 2023-12-24 16:34:08Z flaterco $
# Set unknown datums to estimated MLLW.
# Prerequisites:
# - A tcd file exported from the current state of the database
# - A working tide binary in PATH

require 'pg'
require 'open3'

def doStation(db, hfile, name, comments, index)
  throw "db should not be nil" if db.nil?
  throw "hfile should not be nil" if hfile.nil?
  throw "name should not be nil" if name.nil?
  comments='' if comments.nil?
  throw "index should not be nil" if index.nil?
  puts "Running command for #{name}"
  # The National Tidal Datum Epoch is hardcoded below.
  stdout, stderr, status = Open3.capture3(
    {'HFILE_PATH' => hfile},
    'tide', '-l', name, '-b', '1983-01-01 00:00', '-e', '2002-01-01 00:00',
    '-ms', '-ux')
  if status.success?
    scanrslt = stdout.scan(/Searched interval.*/)
    if scanrslt.empty?
      puts "Failed to find search interval in the output"
      puts "stdout = #{stdout}"
      puts "fixup_datums.rb bailing out"
      exit
    end
    searchline = scanrslt[0]
    throw "searchline should not be nil" if searchline.nil?
    scanrslt = stdout.scan(/Estimated MLLW: *([^ ]+)/)
    if scanrslt.empty?
      puts "Failed to find estimated MLLW in the output"
      puts "stdout = #{stdout}"
      puts "fixup_datums.rb bailing out"
      exit
    end
    datum = -scanrslt[0][0].to_f
    throw "datum should not be nil" if datum.nil?
    comments += "\n" unless comments.empty? or comments[-1] == "\n"
    comments += "MLLW estimated from predictions over National Tidal Datum Epoch\n#{searchline}\n"
    puts "Setting datum to #{datum}"
    res = db.exec("update data_sets set comments = $1, datum = $2, datumkind = 'Approximate Level of Mean Lower Low Water' where index = $3",
      [comments, datum, index])
  else
    puts "Command failed!"
    puts "stdout = #{stdout}"
    puts "stderr = #{stderr}"
    puts "status = #{status}"
    puts "fixup_datums.rb bailing out"
    exit
  end
end

if ARGV.length != 1
  STDERR.puts "Usage: fixup_datums.rb harmonics.tcd"
else
  db = PG::Connection.open(:dbname => "harmbase2")
  query = db.exec("select name, comments, index from data_sets where datumkind = 'Mean Astronomical Tide'")
  stations = query.values
  if stations.empty?
    puts "No stations with missing datums found in database"
  else
    stations.each {|station|
      doStation(db, ARGV[0], station[0], station[1], station[2])
    }
  end
end
