#!/usr/bin/ruby
# $Id: fixup_template.rb 8208 2023-12-28 20:11:18Z flaterco $
# Generate fixups for specified sids.

require 'pg'

def usagedie
  STDERR.puts 'Usage: fixup_template.rb [--state] [--old] sids...'
  exit
end

sids = ARGV
showstate = false
tabname = 'data_sets'
while true
  usagedie if sids.empty?
  if sids[0] == '--state'
    showstate = true
    sids.shift
  elsif sids[0] == '--old'
    tabname = 'data_sets_old'
    sids.shift
  else
    break
  end
end

db = PG::Connection.new(:dbname => 'harmbase2')
db.set_client_encoding 'LATIN1'

def station_exists(db,tabname,sid)
  pattern = sid + '%'
  query = db.exec("select count(*) from #{tabname} where station_id like $1",
                  [pattern])
  !query.getvalue(0,0).to_i.zero?
end
unmatch = false
sids.each{|sid|
  if !station_exists(db,tabname,sid)
    STDERR.puts "Sid prefix #{sid} does not exist in #{tabname}"
    unmatch = true
  end
}
exit if unmatch

query = db.exec("select station_id, name, lat, lng, state from #{tabname} order by station_id, name")
query.each_row {|row|
  sid, name, lat, lng, state = row
  if sids.any?{|p| sid.start_with?(p)}
    mname = db.escape_string(name)
    print "-- #{sid} #{lat} #{lng}\n"
    print "update #{tabname}\n  set "
    if showstate
      print "state = ", (state.nil? ? "null" : "'#{state}'"), ",\n  "
    end
    print "name = '#{mname}'\n  where name = '#{mname}';\n\n"
  end
}
