#!/usr/bin/ruby
# $Id: geogroups_diff.rb 8204 2023-12-26 21:45:48Z flaterco $

require "set"
require "json"
require_relative "../import/importNOS_Util"
require_relative "../import/importNOS_JsonReaders"
require_relative "../import/importNOS_Geogroups"

if ARGV.length != 2
  STDERR.puts "Usage: geogroups_diff.rb old-directory new-directory"
  STDERR.puts "Full paths; each dir must contain geogroups-tides.json."
  exit
end

# readGeogroupsTidesJson reads geogroups-tides.json from working dir.
Dir.chdir(ARGV[0])
gt_old = readGeogroupsTidesJson()
Dir.chdir(ARGV[1])
gt_new = readGeogroupsTidesJson()

ggids_old = gt_old.keys.to_set
ggids_new = gt_new.keys.to_set
ggids_only_old = ggids_old - ggids_new
ggids_only_new = ggids_new - ggids_old
ggids_both = ggids_old & ggids_new

puts "Old = #{ARGV[0]}"
puts "New = #{ARGV[1]}"
ggids_only_old.each{|ggid|
  gold1 = gt_old[ggid]
  gold2 = getTideGeogroupsByGgid(gt_old,ggid)
  puts "Geogroup #{ggid} only in old (disappeared):"
  puts "  Old #{gold1}"
  puts "  Old #{gold2}"
}
ggids_only_new.each{|ggid|
  gnew1 = gt_new[ggid]
  gnew2 = getTideGeogroupsByGgid(gt_new,ggid)
  puts "Geogroup #{ggid} only in new (added):"
  puts "  New #{gnew1}"
  puts "  New #{gnew2}"
}
ggids_both.each{|ggid|
  gold1 = gt_old[ggid]
  gnew1 = gt_new[ggid]
  gold2 = getTideGeogroupsByGgid(gt_old,ggid)
  gnew2 = getTideGeogroupsByGgid(gt_new,ggid)
  if gold1 != gnew1 || gold2 != gnew2
    puts "Geogroup #{ggid} changed:"
    puts "  Old #{gold1}"
    puts "  New #{gnew1}"
    puts "  Old #{gold2}"
    puts "  New #{gnew2}"
  end
}
